=head1 NAME

SDLx::Widget - Common Widgets using SDL

=head1 DESCRIPTION

This distribution provides GUI widgets such as Menu and Textbox.

=head1 USAGE

See the examples directory that came with this package

=head1 BUGS

You tell me! http://sdlperl.ath.cx/

=head1 SUPPORT

 #sdl irc.perl.org

=head1 AUTHORS

    Dustin Mays

    Garu
    CPAN ID: GARU

    Zach
    CPAN ID:ZPMORGAN

	FROGGS
	CPAN ID: FROGGS
    
 
    Kartik Thakore
    CPAN ID: KTHAKORE

    http://sdl.perl.org

=head1 COPYRIGHT

This program is free software; you can redistribute
it and/or modify it under the same terms as Perl itself.

The full text of the license can be found in the
LICENSE file included with this module.


=head1 SEE ALSO

perl(1).

=cut

#################### main pod documentation end ###################


1;
# The preceding line will help the module return a true value

